#!/bin/bash

FILE='/tmp/nisptab.auto'

echo '
This script will help you create the nisptab table file for the
nispero daemon.

-----
NOTE: Before proceeding please plugin the nispero device and connect
it to a serial port in this machine.
-----
'
HOST=`hostname`
echo -n "For which host will we create the table? [$HOST] "
read ANS
if ! [ x"$ANS" == x ] ; then HOST=$ANS ; fi

echo -ne "\nName of the other node(s) in this cluster, space separated: "
read CLUSTER
if [ x"$CLUSTER" == x ] ; then 
	echo "One member cluster? Okey, try again."
	exit
fi


PORT=/dev/ttyS0
echo -ne "\nOn which device is attached the nispero device [/dev/ttyS0] "
read ANS
if ! [ x"$ANS" == x ] ; then PORT=$ANS ; fi
ls1wire $PORT
if [ $? -ne 0 ] ; then
        echo "I didn't find ls1wire, I need this utility to find the chip addresses."
	exit
fi
	
DEVICES=`ls1wire $PORT -c`
# DEVICE='0x00000f0090000008 0x00000f00900000AA'

if [ x"$DEVICES" == x ] ; then
	echo "I didn't find any Nispero device on $DEVICES."
	exit
fi
echo -e "\nI found Nispero device(s)! They are:\n"
i=1;
echo -e '# This configuration file was created by '$0 > $FILE
echo -e '# Configuration file for node $HOST\n' >> $FILE
echo '# Creating aliases (defines) for line_chipN:' >> $FILE
for DM in $DEVICES ; do
	echo "         line_chip"$i with address: $DM
	echo -e "define line_chip"$i "\t0x"$DM >> $FILE
	i=$((i+1))
done
echo -e "\n# Devices:\n# <node name>\t<chip address>\t<outlet(s)>\n" >> $FILE

echo -e "\nEach address can be found on the front panel of the corresponding device.\n"

echo -e "[ $HOST ] can STONITH [ $CLUSTER ]\n"

echo -e "Now we have to map [ $CLUSTER ] to Nispero Device(s) and Outlet(s) \n \
    Nispero Devices are named line_chipN and \n \
    Outlets are enumerated from 1 to 6. \n"

DEV="line_chip1"
for CM in $CLUSTER ; do
	echo -e "Configuring node $CM:"

echo -n $CM "to which device(s) is attached (space separated): [$DEV] "
read ANS
if ! [ x"$ANS" == x ] ; then DEV=$ANS ; fi

for OM in $DEV ; do
	echo -n "to which outlet(s) is $CM connected on $OM (space separated): "
	read ANS
	if [ x"$ANS" == x ] ; then 
		echo "can\'t be empty, you\'ll have to edit the file manually, continuing..."
	fi
	echo -e "$CM\t$OM\t$ANS" >> $FILE
done

done

echo "--- nisptab file ---"
cat $FILE
echo "--- end file ---"

DEST="/etc/nisperod/nisptab"
echo -n "Do you whant to save this file to $DEST [y] "
read ANS
if ! [ "$ANS" = "Y" -o "$ANS" = "y" ] ; then 
	cp -v $FILE $DEST
	if [ $? -eq 0 ] ; then
		rm -f $FILE
		echo "done."
	else
		echo -e "I couldn't copy $FILE to $DEST;\nI won't delete $FILE, so you can copy it manually.\nbye."
	fi
	FILE=$DEST
fi
echo "data file is: $FILE"

