/*
 * Copyright (C) 2004 Andres Sajo
 *
 * This file is part of the nisperod package.
 *
 * nisperod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *
 * $Id: nisperod.h,v 1.19 2004/09/30 13:09:50 andres Exp andres $ 
 */
#ifndef NISPEROD_H
#define NISPEROD_H

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <syslog.h>


/* C++ includes */
/*
#include <iostream.h>
#include <iomanip.h>
#include <unistd.h>
#include <string.h>

#include "1wire/ds2480.h"
#include "1wire/ds2405.h"
#include "1wire/ds2406.h"
#include "1wire/ds2408.h"
*/

#include "wrappers.h"

/* #define DEBUG */ 

#define TRUE 0
#define FALSE 1
#define PIDFILE "/var/run/nisperod.pid"

#define BIT(b) (1 << (b-1))

#define BUFFER 1024

#define CONF "/etc/nisperod/nisperod.cf"
#define TABLE "/etc/nisperod/nisptab"

#define MAX_SOCK 8
/* allowed simultanius connections to daemon, this specifies also the number
 * of simultaneously active children processes.
 */
#define MAX_USERS 8 /* count of ``auth'' keys allowed in config file */

/* Some default values */
/* #define DEFAULT_SOCK_CNT 2 */
#define DEFAULT_SOCKET "/tmp/stonith.socket"
#define DEFAULT_PORT "/dev/ttyS0"

#define SEMKEY	7210934			// semaphor key

/* protocol standards: */
#define PROTO_EXT 16 /* extended protocol magic number */
#define PROTO_SIM 32 /* simple protocol magic number */
#define S_EXTD "extended"
#define S_SMPL "simple"
#define PRTY(num) ((num & PROTO_SIM) ? S_SMPL : S_EXTD)

/* there are magical values */
#define REGISTER "REGISTER:"
#define PASSWORD "PASSWORD:"
#define L_REGISTER 9
#define L_PASSWORD 9


char * gettoken(char *, char *);
extern int found;

struct nispero_t {
	char name[32]; /* gethostname() */
	unsigned long long c_id; /* 1wire chip id */
	unsigned int outlets; /* first six bits */
	struct nispero_t * next;
};

struct def_tab_t{
	char name[33];
	char cid[19]; /* 64 bit in hex value */
	int tag;
	struct def_tab_t * next;
};

struct nispero_t * create_nispero_t_elem(struct nispero_t * follow);
struct nispero_t * destroy_nispero_t_elem(struct nispero_t * elem);
struct def_tab_t * create_def_tab_elem(struct def_tab_t * follow);
struct def_tab_t * destroy_def_tab_elem(struct def_tab_t * elem);
struct nispero_t * seek_device(char * s, struct nispero_t * list);
struct def_tab_t * seek_def(char * s, struct def_tab_t * list);
struct def_tab_t * seek_cid(char * s, struct def_tab_t * list);

int parse_table(const char * table_path);

extern struct nispero_t * nisl_st;
extern struct def_tab_t * defl_st;

int parse_config (const char * config_path);

struct auth_t {
	char login[255];
	char password[255];
	int protocol;
};

struct info_t {
	/* config switches */
	char port[255];
	char socket[255];
	/* int socketcount; */
	struct auth_t users[MAX_USERS]; /* seems more than enought, if not recompile with bigger int */
	/* param switches*/
	int verbose;
};

extern struct info_t info;
extern int usercount;

struct child_pid_t{
	pid_t pid;
	struct child_pid_t * next;
	struct child_pid_t * prev;
};
extern volatile int chldcount;
extern struct child_pid_t * chld_st;

int add_pool(pid_t pid);
int rem_pool(pid_t pid);

void nispero_term(int sig);
void nispero_wait(int sig);
void unixds_term(int sig);
extern volatile int term;

int open_socket(void);
int accept_connect(void);
int close_accept_connect(void);
extern int nis_sock;

#endif
