/* $Id: wrappers.cpp,v 1.5 2004/10/08 14:31:24 andres Exp andres $ */

/*
 * Copyright (C) 2004 Andres Sajo
 *
 * This file is part of the nisperod package.
 *
 * nisperod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */
 
#include <iostream.h>
#include <iomanip.h>
#include <unistd.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <time.h>

#include "wrappers.h"
#include "nisperod.h"

#include "ds2480.h"
#include "ds18s20.h"
#include "ds2404.h"
#include "ds2405.h"
#include "ds2406.h"
#include "ds2408.h"
#include "ds2433.h"
#include "ds2438.h"
#include "ds2890.h"
#include "device.h"

using namespace ::OneWire;

DS2480 bus;

void turn_signal(bool turn_on, Address add);
extern  int semid;

int wrapper_open_dev(char * d) {
	struct sembuf operations[1];
   operations[0].sem_num = 0;
	operations[0].sem_op = -1;
	operations[0].sem_flg = 0;
	int semret = -1;
	time_t etime;
	etime = time(0) + 5;
	do {
		semret = semop(semid,operations,1);
		if(etime < time(0)) return 1;
	} while(semret != 0);

	if (bus.Open(d) == false){
		return 1;
	} else {
		return wrapper_reset_dev();
	}
}

int wrapper_reset_dev(void) {
	return wrapper_status();
}

int wrapper_compare_tables(void) {
	int i, count;
	string ic;
	struct def_tab_t * found;
	
	bus.FindDevices();
	count = bus.GetDeviceCount();
	for(i = 0; i < count; i++)
	{
#ifdef DEBUG
		cerr << i + 1 << ":  " << bus.GetDevice(i)->GetAddress().GetString() << "\t";
		cerr << bus.GetDevice(i)->GetType() << endl;
#endif
		
		if (bus.GetDevice(i)->GetType() == IC)
		{
#ifdef DEBUG
			cerr << "found: " << bus.GetDevice(i)->GetAddress().GetString() << endl;
#endif
			ic = "0x" + bus.GetDevice(i)->GetAddress().GetString();
			// find and tag this define node:
			
			found = seek_cid((char *)ic.c_str(), defl_st);
			if (found == NULL)
			{
				cerr << "IC address " << ic
				<< " is not defined in " << TABLE << endl;
				return 1;
			}
			if (found->tag == TRUE)
			{
				cerr << "IC address " << ic
				<< " is already defined in " << TABLE << endl;
				return 1;
			}
			found->tag = TRUE;
		}
	}
	// see if there is a define that wasn't taged:
	found = defl_st;
	while (found != NULL)
	{
#ifdef DEBUG
		printf("%p: found->name(%s), found->tag(%i) -> %p\n", found, found->name, found->tag, found->next);
#endif
		if (found->tag == FALSE){
			cerr << "some extra defines in " << TABLE << " where not mapped to" << endl
			<< "phisical devices, reconfigure define table." << endl;
			return 1;
		}
		found = found->next;
	}
	return 0;
}


int wrapper_close_dev(void){
	bus.Close();

	struct sembuf operations[1];
   operations[0].sem_num = 0;
	operations[0].sem_op = 1;
	operations[0].sem_flg = 0;
	int semret = -1;
	time_t etime;
	etime = time(0) + 5;
	do {
		semret = semop(semid,operations,1);
		if(etime < time(0)) return 1;
	} while(semret != 0);
	return 0;
}

int wrapper_status(){
	if(bus.Reset() == false) return 1;
	return 0;
}

int wrapper_turn_off(char * device){
	struct nispero_t * nis;
	char address[20];
	int ok = 1;
	string s;
	unsigned char var = 0;
	/* Check out why issue 2 */
	/* if(bus.Reset() == false) return 1; */
	nis = seek_device(device, nisl_st);
	/* printf("--> ON:after bus.Reset, and nis = %p\n", nis); */
	while (nis != NULL){
		ok = 0;
		Address add;
		sprintf(address, "%016Lx", nis->c_id);
		add.FromString(address);
		bus.SelectDevice(add);
		DS2408 sw(&bus, add);
#ifdef USE_SIGNAL_LED
		turn_signal(true, add);
#endif
		var = sw.ReadOutputLatch();
		var = sw.ReadOutputLatch();
		/* DEBUG ***
		printf("in turn Off: var = 0x%02x, add = \'%s\', outlets: 0x%02x", var, address, nis->outlets);
		*/
		var &= ~nis->outlets;
		/* DEBUG ***
		printf("in turn Off: var = 0x%02x, add = \'%s\', outlets: 0x%02x", var, address, nis->outlets); 
		*/
		sw.SetOutputLatch(var);
		s = add.GetString();
		// 016Lx
		/* printf(", var = 0x%02x, add = \'%s\'\n", var, s.c_str()); */
		nis = seek_device(device, nis->next);
#ifdef USE_SIGNAL_LED
		turn_signal(false, add);
#endif
	}
	return ok;
}

int wrapper_turn_on(char * device){
	struct nispero_t * nis;
	char address[20];
	int ok = 2;
	string s;
	unsigned char var = 0;
	/* if(bus.Reset() == false) return 1; */
	nis = seek_device(device, nisl_st);
	while (nis != NULL){
		ok = 0;
		Address add;
		sprintf(address, "%016Lx", nis->c_id);
		add.FromString(address);
		bus.SelectDevice(add);
		DS2408 sw(&bus, add);
#ifdef USE_SIGNAL_LED
		turn_signal(true, add);
#endif
		var = sw.ReadOutputLatch();
		var = sw.ReadOutputLatch();
		/* DEBUG ***
		printf("in turn On: var = 0x%02x, add = \'%s\', outlets: 0x%02x", var, address, nis->outlets);
		*/
		var |= nis->outlets;
		/* DEBUG ***
		printf("in turn On: var = 0x%02x, add = \'%s\', outlets: 0x%02x", var, address, nis->outlets);
		*/
		sw.SetOutputLatch(var);
		s = add.GetString();
		// 016Lx
		/* printf(", var = 0x%02x, add = \'%s\'\n", var, s.c_str()); */
		nis = seek_device(device, nis->next);
#ifdef USE_SIGNAL_LED
		turn_signal(false, add);
#endif
	}
	return ok;
}

int wrapper_get(struct def_tab_t * def){
	const char * tmp = &def->cid[2];
	Address add;
	
/* 	if(bus.Reset() == false) return -1; */
	
	add.FromString(tmp);
	
	bus.SelectDevice(add);
	DS2408 sw(&bus, add);

	// sw.ToggleOutputBit(0);
	
	return sw.ReadOutputLatch();
}

int wrapper_set(struct def_tab_t * def, int i){
	const char * tmp = &def->cid[2];
	Address add;
/*	if(bus.Reset() == false) return -1;*/

	add.FromString(tmp);

	bus.SelectDevice(add);
	DS2408 sw(&bus, add);
	sw.SetOutputLatch(i);

	return sw.ReadOutputLatch();
}

void turn_signal(bool turn_on, Address add){
#ifdef USE_SIGNAL_LED
	unsigned char var;
	bus.SelectDevice(add);
	DS2408 sw(&bus, add);
	var = sw.ReadOutputLatch();
	
	if (turn_on == false){
		var |= SIGNAL_LED;
	} else {
		var &= ~SIGNAL_LED;
	}
	
	sw.SetOutputLatch(var);
	var = sw.ReadOutputLatch();
#endif
	return;
}

#if 0
int main( int argc, char* argv[] )
{
	DS2480 bus;
	if( argc != 2 )
	{
		cout<<argv[0]<<" [port]"<<endl;
		return 1;
	}
	
	if(bus.Open(argv[1]) == false)
	{
		cout<<"Couldn't open the 1-wire bus!"<<endl;
		return 1;
	}
	
	if(bus.Reset() == false)
	{
		cout<<"Couldn't reset the 1-wire bus!"<<endl;
		return 1;
	}
	
	bus.FindDevices();
	int count = bus.GetDeviceCount();
	for(int i=0;i<count;i++)
	{
		cout<<i+1<<":  "<<bus.GetDevice(i)->GetAddress().GetString() <<"\t";
		cout<<bus.GetDevice(i)->GetType()<<endl;
	}
	bus.Close();
	return 0;
}
#endif
